/*********************************************************/
/***              STRYK System Improvement             ***/
/***    Performance Optimization & Troubleshooting     ***/
/***  (c) 2007, STRYK System Improvement, Jrg Stryk   ***/
/***                   www.stryk.info                  ***/
/*********************************************************/

if exists (select * from dbo.sysobjects 
where id = object_id(N'[dbo].[ssi_delzerosift]') and objectproperty(id, N'IsProcedure') = 1)
drop procedure [dbo].[ssi_delzerosift]
go

create procedure dbo.ssi_delzerosift
  @nomsg tinyint = 0,    -- Hides info-messages
  @script tinyint = 0    -- Create just TSQL script 
with encryption
as

print '---------------------------------------------------------'
print '---              STRYK System Improvement             ---'
print '---    Performance Optimization & Troubleshooting     ---'
print '---  (c) 2007, STRYK System Improvement, Jrg Stryk   ---'
print '---                   www.stryk.info                  ---'
print '---------------------------------------------------------'
print '---           Version 1.06, Date: 11.06.2007          ---'
print ''

if (@nomsg not in (0,1)) begin
  raiserror ('ERROR: Invalid Parameter @nomsg: %i', 15, 1, @nomsg)
  return
end
if (@script not in (0,1)) begin
  raiserror ('ERROR: Invalid Parameter @script: %i', 15, 1, @script)
  return
end

declare @sift_tab_id int, 
        @fld_name varchar(20), 
        @statement nvarchar(4000), 
        @fldno_total int, 
        @fldno int,
        @fld_size int,
        @rec_size bigint,
        @rec_curr bigint,
        @rec_total bigint,
        @rec_total_size bigint

set @rec_total = 0
set @rec_total_size = 0

set nocount on
set statistics io off
set transaction isolation level serializable

if @script = 1 begin
  print '/* Begin >> */'
  print 'USE [' + db_name() + ']'
  print 'GO'
  print 'SET TRANSACTION ISOLATION LEVEL SERIALIZABLE'
end

-- temporary table for sorting by table-name
create table #sift_tab ([tab_id] int, [tab_name] varchar(250))
insert into #sift_tab select [id], object_name([id]) from dbo.sysobjects where 
  (objectproperty([id], N'IsUserTable') = 1) and (object_name([id]) like ('%$' + '[0-9]' + '%')) and right(object_name([id]), 1) in ('0','1','2','3','4','5','6','7','8','9')

-- cursor SIFT tables
declare sift_tab_cur cursor fast_forward for select [tab_id] from #sift_tab order by [tab_name]
open sift_tab_cur
fetch next from sift_tab_cur into @sift_tab_id
while @@fetch_status = 0 begin
  -- check for valid SIFT record
  if (not exists(select [name] from dbo.syscolumns where (id = @sift_tab_id) and ([name] like 's' + '[0-9]' + '%')))
    or
     (not exists(select [name] from dbo.syscolumns where (id = @sift_tab_id) and ([name] like 'f' + '[0-9]' + '%')))
    or
     (exists(select [name] from dbo.syscolumns where (id = @sift_tab_id) and ([name] like 's' + '[0-9]' + '%') and (type_name([xtype]) <> 'decimal')))
    or
     (exists(select [name] from dbo.syscolumns where (id = @sift_tab_id) and ([name] = 'timestamp')))
    goto End_Step

  -- get size of SIFT record
  select @rec_size = sum([length]) from dbo.syscolumns where id = @sift_tab_id

  -- create statement
  set @statement = 'DELETE FROM dbo."' + object_name(@sift_tab_id) + '" WITH(TABLOCKX,HOLDLOCK) WHERE ('
  select @fldno_total = count([id]) from dbo.syscolumns where id = @sift_tab_id and [name] like 's%'
  set @fldno = 1

  -- cursor SIFT sum fields
  declare fld_cur cursor fast_forward for select [name] from dbo.syscolumns where id = @sift_tab_id and [name] like 's%'
  open fld_cur
  fetch next from fld_cur into @fld_name
  while @@fetch_status = 0 begin
    set @statement = @statement + '(' + @fld_name + ' = 0)'
    if (@fldno >= 1) and (@fldno < @fldno_total) begin
      set @statement = @statement + ' AND '
    end
    set @fldno = @fldno + 1    
    fetch next from fld_cur into @fld_name
  end
  close fld_cur
  deallocate fld_cur
  set @statement = @statement + ')'
  if @script = 1
    print @statement
  else begin
    if @nomsg = 0
      print 'Cleaning SIFT table "' + object_name(@sift_tab_id) + '" ...'
    begin transaction
    exec(@statement)
    set @rec_curr = @@rowcount
    if @@trancount > 0 commit transaction
    if @nomsg = 0
      print convert(varchar(15), @rec_curr) + ' records deleted (' + convert(varchar(15), (@rec_size * @rec_curr)) + ' bytes)'
    set @rec_total = @rec_total + @rec_curr
    set @rec_total_size = @rec_total_size + (@rec_size * @rec_curr)
  end  
  End_Step:
  fetch next from sift_tab_cur into @sift_tab_id
end
close sift_tab_cur
deallocate sift_tab_cur
drop table #sift_tab
if @@trancount > 0 commit transaction
if @script = 0
  print 'Finished. Total number of deleted "Zero SIFT" records: ' + convert(varchar(15), @rec_total) + ' (' + convert(varchar(15), @rec_total_size) + ' bytes)'
else
  print '/* << End */'
select @rec_total as [records], @rec_total_size as [bytes] 
